#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = __importDefault(require("commander"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const inquirer_1 = __importDefault(require("inquirer"));
const shelljs_1 = __importDefault(require("shelljs"));
const util_1 = __importDefault(require("util"));
const command_exists_1 = require("command-exists");
const os_1 = __importDefault(require("os"));
const package_json_1 = __importDefault(require("../../package.json"));
const getOptions_1 = __importDefault(require("../utils/getOptions"));
const spinner_1 = __importStar(require("../utils/spinner"));
const logger_1 = require("../utils/logger");
const cwd = process.cwd();
// 自动版本检测
require('../scripts/check_latest');
process.on('unhandledRejection', err => {
    throw err;
});
commander_1.default
    .version(package_json_1.default.version)
    .option('-P, --proxy <proxy>', '设置代理')
    .option('-x, --defaultProxy', '使用内网代理', false);
commander_1.default
    .command('create [dir]')
    .description('初始化项目')
    .action((dir, options) => {
    require('../commands/create')(dir, options);
});
commander_1.default
    .command('dev')
    .description('启动开发者模式')
    .option('-s, --ssr', '服务端渲染开发调试')
    .option('-p, --port <port>', '配置开发者服务器监听端口')
    .option('--cache-dir <dir>', '编译阶段缓存目录,加速二次编译')
    .option('--no-eslint', '禁用eslint检测代码')
    .option('-c, --css-source-map', '使用cssSourceMap ，但会导致开发模式 FOUC')
    .action(options => {
    require('../commands/dev')(options);
});
commander_1.default
    .command('build')
    .description('构建生产包')
    .option('-d, --dist <dist>', '配置构建文件生成目标目录')
    .option('-a, --analyzer', '开启构建分析', false)
    .option('-m, --use-smp', '分析构建耗时', false)
    .option('-s, --source-map', '是否生成source-map,默认false', false)
    .option('--no-mini', '禁用压缩代码')
    .option('--cache-dir <dir>', '编译阶段缓存目录,加速二次编译')
    .option('--no-silent', '输出日志')
    .option('--dev', '环境变量使用development')
    .action(options => {
    require('../commands/build').default(options);
});
commander_1.default
    .command('test')
    .description('运行 jest 测试')
    .option('--coverage', 'coverage')
    .option('--watchAll', 'watch')
    .option('--env', 'environment')
    .action(options => {
    require('../commands/test')(options);
});
commander_1.default
    .command('check')
    .description('检测代码是否合并主干')
    .action(options => {
    require('../commands/check.js')(options);
});
commander_1.default
    .command('clean')
    .description('清理缓存文件')
    .action(async () => {
    spinner_1.default.logWithSpinner('清理缓存');
    const options = getOptions_1.default({});
    await fs_extra_1.default.emptyDir(options.cache);
    spinner_1.default.stopSpinner();
});
commander_1.default
    .command('cache <cmd>')
    .description('缓存node_modules加速构建,使用方法：imt cache [cmd]. cmd 是配置的 npm scripts key')
    .action(async (cmd) => {
    const options = getOptions_1.default({});
    const cache = require('../scripts/cache');
    cache({ cmd, cache: options.cache });
});
const initChoices = [
    { name: '添加 提交前 lint 和 prettier', value: 'lint' },
    { name: '添加 commit msg规范检测', value: 'commit' },
];
commander_1.default
    .command('add [type]')
    .description('添加项目配置,支持:lint,commit')
    .action(async (type, options) => {
    if (!type) {
        ({ type } = await inquirer_1.default.prompt([
            {
                type: 'list',
                name: 'type',
                message: '选择初始化内容！',
                choices: initChoices,
            },
        ]));
    }
    const choice = initChoices.find(i => i.value === type);
    if (choice) {
        const pkgFile = path_1.default.join(cwd, 'package.json');
        const pPkg = require(pkgFile);
        shelljs_1.default.cd(cwd);
        let npmCmd = 'npm';
        if (command_exists_1.sync('tnpm')) {
            npmCmd = 'tnpm';
        }
        switch (type) {
            case 'lint':
                pPkg['lint-staged'] = {
                    '*.{json,css,scss,md}': ['prettier --write', 'git add'],
                    '*.{jsx,js}': ['prettier --write', 'eslint --fix', 'git add'],
                };
                pPkg.husky = pPkg.husky || {};
                pPkg.husky.hooks = pPkg.husky.hooks || {};
                pPkg.husky.hooks['pre-commit'] = 'lint-staged';
                spinner_1.logWithSpinner('添加配置信息');
                fs_extra_1.default.writeFileSync(pkgFile, JSON.stringify(pPkg, null, 2));
                spinner_1.logWithSpinner('安装依赖：husky,prettier,lint-staged');
                await util_1.default.promisify(shelljs_1.default.exec)(`${npmCmd} i husky prettier lint-staged -D`, { silent: true });
                spinner_1.stopSpinner();
                break;
            case 'commit': {
                spinner_1.logWithSpinner('安装依赖：commitlint-config-imt');
                await util_1.default.promisify(shelljs_1.default.exec)(`${npmCmd} i commitlint-config-imt -D`, { silent: true });
                spinner_1.logWithSpinner('初始化commit配置');
                const cmd = `./node_modules/.bin/imt-commit${os_1.default.platform() === 'win32' ? '.cmd' : ''}`;
                await util_1.default.promisify(shelljs_1.default.exec)(cmd, { silent: true });
                spinner_1.stopSpinner();
                break;
            }
            default:
        }
    }
    else {
        logger_1.error(`不支持该选项: ${type}`);
        options.outputHelp();
    }
});
commander_1.default.command('*').action(options => {
    logger_1.error(`找不到命令: ${options}`);
    commander_1.default.outputHelp();
});
commander_1.default.parse(process.argv);
if (!process.argv.slice(2).length) {
    commander_1.default.outputHelp();
}
