"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const webpack_1 = __importDefault(require("webpack"));
const index_js_1 = __importDefault(require("../config/webpack/index.js"));
const const_1 = require("../const");
const index_js_2 = __importDefault(require("../index.js"));
const getOptions_1 = __importDefault(require("../utils/getOptions"));
const logger_1 = require("../utils/logger");
const spinner_1 = require("../utils/spinner");
process.env.NODE_ENV = const_1.PROD;
async function buildSSR(options, imt) {
    logger_1.info('build ssr');
    options.type = const_1.SSR;
    const { hooks } = imt;
    await new Promise(resolve => {
        hooks.beforeSSRBuild.callAsync(imt, resolve);
    });
    fs_extra_1.default.emptyDirSync(options.ssrConfig.dist);
    fs_extra_1.default.emptyDirSync(options.ssrConfig.view);
    spinner_1.logWithSpinner('clean ssr dist dir.');
    spinner_1.stopSpinner();
    await new Promise(resolve => {
        const webpackConfig = index_js_1.default(options);
        webpack_1.default(webpackConfig, (err, stats) => {
            if (err) {
                console.log(err);
                process.exit(1);
            }
            if (stats.hasErrors()) {
                process.exit(1);
            }
            resolve();
        });
    });
    await new Promise(resolve => {
        hooks.afterSSRBuild.callAsync(imt, async () => {
            resolve();
        });
    });
}
exports.default = async (argv) => {
    logger_1.info('build frontend');
    const options = getOptions_1.default(argv);
    if (options.dev) {
        process.env.NODE_ENV = const_1.DEV;
    }
    options.type = const_1.PROD;
    const imt = new index_js_2.default(options);
    const { silent } = options;
    if (silent) {
        process.noDeprecation = true;
    }
    await new Promise(resolve => {
        imt.hooks.beforeBuild.callAsync(imt, resolve);
    });
    spinner_1.logWithSpinner('clean frontend dist dir.');
    spinner_1.stopSpinner();
    fs_extra_1.default.emptyDirSync(options.dist);
    await new Promise(resolve => {
        const webpackConfig = index_js_1.default(options);
        webpack_1.default(webpackConfig, (err, stats) => {
            if (err) {
                logger_1.error(err);
                process.exit(1);
            }
            if (stats.hasErrors()) {
                process.exit(1);
            }
            resolve();
        });
    });
    await new Promise(resolve => {
        imt.hooks.afterBuild.callAsync(imt, async () => {
            resolve();
        });
    });
    if (options.ssrConfig) {
        await buildSSR(options, imt);
    }
};
